#!/usr/bin/env wish

# Copyright (c) 2002-2008, Botik-Technologies, Ltd.
# See license.txt for more information.

# ================= Client =========================================================
# Предоставляет
#
#     startClient { }
#     restartClient { {errLevel 1} }
#     writeLogChangeStatus { level typeState text }
#
# Использует
#
#     $botikkey(port)
#     $botikkey(d_password)
#     $botikkey(region_request)
#     $botikkey(region_by_server)
#     $botikkey(region_by_user)
#
# Параметры и переменные
#
global bkClientRestartTime bkClientTimeOut bkClientPings bkClientSuccPings bkClientExpected bkClientSock bkClientExpectedRegion

set    bkClientRestartTime    3 ; # в секундах, между перезапусками клиента
set    bkClientTimeOut       60 ; # в секундах, TimeOut ожидания любой иной посылки от сервера
set    bkClientPings          3 ; # Сколько ping-тестов будем делать перед открытием сокета и ...
set    bkClientSuccPings      1 ; # ... сколько из этого достаточно иметь удачных ping-тестов
set    bkClientExpected      { }; # Какие сообщения ждем (допускаем) от Кей-сервера
set    bkClientSock          "" ; # Сокет (если открыт)
set    bkClientExpectedRegion WORLD ; # Запрошенный регион

# Выполнение ping-тестирования Кей-сервера перед открытием сокета
# $bkClientPings попыток, до $bkClientSuccPings удач.
# Возвращает 0 -- если удалось, число неудач -- иначе.

proc pingTest { host } {
   global bkClientPings bkClientSuccPings
   set lost 0
   set rec  0
   for {set i 1} {$i <= $bkClientPings} {incr i} {
      array set ping [rawPing $host] ; # умолчательные size=1450 timeout=250
      if {$ping(rec) == 1} {
         incr rec
         if {$rec >= $bkClientSuccPings} {
            return 0
         }
      } else {
         incr lost
      }
   }
   return $lost
}

proc restartClient {{errLevel 1}} {
   global bkClientRestartTime botikkey
   catch { disableReadableSocketHandler }
   catch { close $bkClientSock }
   set botikkey(client_started)   0
   set botikkey(region_by_server) NONE
   writeLogChangeStatus 0 RED "[strTime] Остановили клиента"
   if {$errLevel > 0} {
      forceLogWindow
   }
   return []
}

proc startClient { } {
   global botikkey bkClientSock bkClientPings

   if {$botikkey(addr) eq ""} {
   # Если IP Key-сервера не был вручную прописан в настройках:
      set ip [rawGateway]
      set botikkey(addr) $ip
   } else {set ip $botikkey(addr)}

   if {$ip eq -1} {
      writeLogChangeStatus 1 RED "[strTime] Ошибка: не удалось установить IP-номер шлюза (rawGateway == -1)"
      restartClient
      return [ ]
   }
   if {[windowsPlatform]} {
      set lost [pingTest $ip]
      if {$lost > 1} {
         writeLogChangeStatus 1 RED \
            "[strTime] Ошибка: Связь с Кей-сервером \[$ip\] невозможна: из $bkClientPings ping-тестов $lost неудачных"
         restartClient
         return [ ]
      } else {
         writeLogChangeStatus 3 YELLOW \
            "[strTime] Успешные ping-тесты \[$ip\] ([expr $bkClientPings-$lost] из $bkClientPings)"
      }
   }
   writeLogChangeStatus 2 GRAY "[strTime] Устанавливаем соединение -- open socket \[$ip:$botikkey(port)\]..."
   set ok 0
   catch {
      catch {close $bkClientSock}
      set sock [socket $ip $botikkey(port)]
      fconfigure $sock -encoding koi8-r
      set ok 1
   } rc
   if {!$ok} {
      catch {close $sock}
      writeLogChangeStatus 1 RED "[strTime] Ошибка: не удалось установить соединение -- [mkErrorCode $rc ", "]"
      restartClient
      return [ ]
   }
   writeLogChangeStatus 1 YELLOW "[strTime] Соединение установлено"
   set botikkey(client_started) 1
   set botikkey(time_last_read) [clock seconds]
   set bkClientSock $sock
   setExpected {BOTIKKEY}
   enableReadableSocketHandler
   return [ ]
}

proc enableReadableSocketHandler { } {
   global bkClientSock
   catch {fileevent $bkClientSock readable {readSocketHandler}}
   after 100
   return [ ]
}

proc disableReadableSocketHandler { } {
   global bkClientSock
   fileevent $bkClientSock readable ""
   return [ ]
}

proc readSocketHandler { } {
   global botikkey bkClientSock bkClientExpectedRegion bkClientTimeOut

   array set region2collor { WORLD GREEN    LOCAL GREEN     NONE RED }
   disableReadableSocketHandler ; # ----------------- Блокировали "readSocketHandler"

   set ok 0
   catch {
      set eof_sock [eof $bkClientSock]
      set ok 1
   } rc
   if { ! $ok } {
      writeLogChangeStatus 1 YELLOW "[strTime] Ошибка: не удалось выполнить проверку \"eof\" -- [mkErrorCode $rc ", "]"
      restartClient
      return 0
   }
   if { $eof_sock } {
      writeLogChangeStatus 1 YELLOW "[strTime] Ошибка: во время чтения с Кей-сервера получено \"end of file\""
      restartClient
      return 0
   }

   set ok 0
   catch {
      set text [gets $bkClientSock]
      set ok 1
   } rc
   if { ! $ok } {
      writeLogChangeStatus 1 YELLOW "[strTime] Ошибка: не удалось прочитать с Кей-сервера -- [mkErrorCode $rc ", "]"
      restartClient
      return 0
   }

   set botikkey(time_last_read) [clock seconds]

   set tag UnKnown
   regexp {^(CHALLENGE|MESSAGE|BOTIKKEY|GOOD|SORRY) *(.*)[\r\n]*$} $text skip tag txt
   if {$tag eq "CHALLENGE" || $tag eq "BOTIKKEY"} {
   } else {
   }
   set expected [nowExpected]
   
   if { [lsearch -exact $expected $tag] < 0 } {
      writeLogChangeStatus 1 YELLOW \
            "[strTime] Ошибка: protocol error -- ожидали от Кей-сервера \{$expected\}, a получено: $text"
      restartClient
      return
   } else {
      switch $tag {
         BOTIKKEY  {
            writeLogChangeStatus 0 YELLOW "[strTime] Соединились с Кей-сервером: $text"
            set time 60
            catch {
               if [regexp "TIMEOUT=(.+)" $text skip time1] {
                  set time1 [expr int($time1)]
                  if {$time1 > 0} {
                     set time $time1
                  }
               }
            }
            set bkClientTimeOut $time
            writeLogChangeStatus 2 GRAY "[strTime] Предел ожидания сообщений от Кей-сервера: $time сек"
            setExpected {CHALLENGE MESSAGE}
         }
         MESSAGE   {
            set str "[strTime] Поступило сообщение от СТ \"$::net(name)\":\n$txt"
            writeLogChangeStatus 0 BLUE $str
            forceLogWindow 1
            setExpected [nowExpected]
            # nowExpected остается неизменным
         }
         CHALLENGE {
            writeLogChangeStatus 2 GRAY "[strTime] Сервер выдал запрос (challenge)..."
            set r1 0.0
            set r2 0.0
            set r3 0.0
            catch {set r1 [expr rand()]}
            catch {set r2 [expr rand()]}
            catch {set r3 [expr rand()]}
            set random                 [string tolower [::md5::md5 -hex -- "${r1}-${r2}-$r3"]]
            set bkClientExpectedRegion $botikkey(region_request) ; # WORLD | LOCAL | NONE
            set t "$txt$random$bkClientExpectedRegion$botikkey(d_password)"
            set digest [string tolower [::md5::md5 -hex -- $t]]
            writeLogChangeStatus 2 GRAY \
               "[strTime] Посылаем наш ответ (response), запрошен доступ $bkClientExpectedRegion ..."
            set ok 0
            catch {
               set msg "RESPONSE $digest $random $bkClientExpectedRegion\n"
               puts -nonewline $bkClientSock $msg
               flush $bkClientSock
               set ok 1
            } rc
            if { ! $ok } {
               writeLogChangeStatus 1 YELLOW \
                  "[strTime] Ошибка: посылка ответа не удалась -- [mkErrorCode $rc ", "]"
               restartClient
               return 0
            }
            setExpected {GOOD SORRY MESSAGE}
         }
         GOOD      {
            # А как с запрошенной/полученой зоной доступа?
            set region $bkClientExpectedRegion
            regexp {^(WORLD|LOCAL|NONE)} $txt skip region
            set collor YELLOW
            catch { set collor $region2collor($region) }
            set botikkey(region_request) [set botikkey(region_by_server) $region]

            if {$region ne $bkClientExpectedRegion} {
               # запрошен один регион $bkClientExpectedRegion, получен иной: $region
               writeLogChangeStatus 0 $collor "[strTime] Запросили $bkClientExpectedRegion,\
                                              наш ответ признан, но выдан доступ $region,\
                                              Кей-сервер сказал: $text"
               set botikkey(region_by_user)   $region
               forceLogWindow
            } else {
               writeLogChangeStatus 0 $collor "[strTime] Наш ответ признан, Кей-сервер сказал: $text"
            }
            setExpected {CHALLENGE MESSAGE}
         }
         SORRY     {
            writeLogChangeStatus 0 RED "[strTime] Наш ответ отвергнут, Кей-сервер сказал: $text"
            if { $botikkey(region_by_server) ne "NONE" } {
               set botikkey(region_by_server) "NONE"
               forceLogWindow
            }
            setExpected {CHALLENGE MESSAGE}
         }
      }
   }
  enableReadableSocketHandler
  return 1
}

proc setExpected { list } {
   global bkClientExpected
   set bkClientExpected $list
   return []
}

proc nowExpected { } {
   global bkClientExpected
   return $bkClientExpected
}

proc writeLogChangeStatus { level typeState text } {
   dolog $text
   writeLog $level $typeState $text
   if { [lsearch -exact { RED YELLOW GREEN } $typeState] >= 0 } {
      changeStatus $typeState $text
   }
   return []
}

proc debug {msg} {dolog "[strTime] $msg"}
# Журналирование
proc dolog {msg} {
    if {![file writable .]} {return}
    set filename $::botikkey(log)
    catch {
    set f [open $filename a]
    puts $f [string trimright "$msg" "\n"]
    close $f
    if {[file size $filename] >= 1024*1024} {
        file delete -force $filename
    }
    }
}
proc ftime {} {
    return [clock format [clock seconds] -format "%Y-%m-%d %X"]
}
proc clientMainLoop {} {
    proc checkIt {} {
        if {!$::botikkey(client_started)} {
            writeLogChangeStatus 3 GRAY "[strTime] Запускаем клиента"
            startClient
        } elseif {[clock seconds] - $::botikkey(time_last_read) > $::bkClientTimeOut} {
            writeLogChangeStatus 0 RED "[strTime] Истёк период ожидания сообщений от сервера"
            restartClient
        }
        after [expr 1000*$::botikkey(restart_check_timeout)] checkIt
    }
    checkIt
}
