#!/bin/sh\
exec wish "$0" "$@"

# Copyright (c) 2003-2008, Botik-Technologies, Ltd.
# See license.txt for more information.

proc initValues {} {
   global botikkey whoami positions

   # Версия, дата текущей версии и начало разработки:
   set botikkey(name)       "Botik Key"             ;  # имя
   set botikkey(ver)        1.45                    ;  # версия программы
   set botikkey(ver_date)   "22 октября 2008 года"   ;  # дата выпуска версии

   set botikkey(addr)       "";                        # адрес Кей-серверa, вводимый вручную на случай, если не удалось узнать IP шлюза
   set botikkey(port)       12040   ;                  # порт

   set botikkey(d_password) [pswdToKey  ""] ;          # закрученный пароль
   set botikkey(md5_empty)  $botikkey(d_password);     # константа: пустой закрученный пароль
   set botikkey(save_pswd) 1 ;                         # 1 == вводить с клавиатуры; 2 == через переменную среды;
                                                       # 3 == хранить в умолчательном файле; 4 == хранить в указанном файле
   set botikkey(user_pswd_file) "**файл не указан**" ; # пользовательский файл для хранения пароля
   set botikkey(dflt_pswd_file) [appFile botik_key.pswd]; # умолчательный файл для хранения пароля
   set botikkey(init_file)      [appFile botik_key.init]; # файл с опциями
   
   set botikkey(verbose)    1 ;             # подробность журнала (0,1,2,3)
   set botikkey(minimized)  1 ;             # стартовать как иконка? (0 1)
   set botikkey(showLog)    0 ;             # для важных сообщений форсировать показ журнала (0 1)
   set botikkey(icontype)   2 ;             # тип иконок (0, 1, 2 -- ниже)
   set botikkey(iconwindow) .icon
   #
   # Картинки для иконок (в зависимости от типа $botikkey(icontype)):
   #
   #                  не-Windows: 0          1 - Windows               2 - Windows + плохое зрение 
   set botikkey(RED)         {img/botik_none.gif    img/botik_red.ico         img/red.ico}
   set botikkey(YELLOW)      {img/botik_wait.gif    img/botik_yellow.ico      img/yellow.ico}
   set botikkey(GREEN)       {img/botik_world.gif   img/botik_green.ico       img/green.ico}
   set botikkey(GREEN_LOCAL) {img/botik_local.gif   img/botik_green_local.ico img/green_local.ico}

   array set positions {img/green.ico 0 img/green_local.ico 1 img/red.ico 2 img/yellow.ico 3 img/botik_green.ico 0 img/botik_green_local.ico 1 img/botik_red.ico 2 img/botik_yellow.ico 3}
   set botikkey(modenames) {WORLD "Внешний доступ" LOCAL "Локальный доступ" NONE "Нет доступа"}
   #
   if {[windowsPlatform]} {
       set botikkey(currentIcon) [lindex $botikkey(YELLOW) 2]
   } else {
       set botikkey(currentIcon) [lindex $botikkey(YELLOW) 0]
   }
   set botikkey(timeForNextShowMessage) [expr [clock seconds]-10] ;
   #
   set botikkey(start_date) "19 января 2003 года"             ; # дата начала разработки
   set botikkey(url)        http://www.botik.ru/~botik/key/   ; # официальный сайт программы
   set botikkey(url_tools)  http://www.botik.ru/~botik/tools/ ; # официальный сайт программы
   set botikkey(url_dload)  http://www.$::net(domain)/~botik/tools/download.ru.html ; # нов.версия
   set botikkey(url_NadminAbPswd) http://nadmin.$::net(domain)      ; # где устанавливают пароли Абоненты
   set botikkey(version) "$botikkey(name) v.$botikkey(ver)"   ; # имя и версия программы
   set botikkey(region_init)       LOCAL   
   set botikkey(region_by_user)    LOCAL
   set botikkey(region_request)    LOCAL
   set botikkey(region_by_server)  NONE
   set botikkey(currentUColorName) YELLOW
   set botikkey(client_started)    0
   set botikkey(chk_version)       1
   set botikkey(url_chk_version)   http://www.$::net(domain)/~botik/download/versions.txt
   set botikkey(url_whoami)        https://nadmin.$::net(domain)/~nadmin/whoami.cgi
   set botikkey(time_last_read)    0
   set botikkey(restart_check_timeout) 3; # timeout (в сек.) на проверку "нужен ли перезапуск клиента"
   set botikkey(log) [appFile botikkey.log 1]
   #
   # Будет сделано [readURL $botikkey(url_chk_version) 10000], когда будет
   # ($botikkey(currentUColorName)==GREEN) && $botikkey(chk_version)
   
   return []
}

proc analyzeDataFromUrl { url data } {
   global botikkey

   switch -regexp $url {
      versions.txt {
         # -------------------------------------------------- versions.txt
         set tokens [scolSplit $data]
         foreach {pgm ver} $tokens {
            if [string is double -strict $ver] {
               set versions_http($pgm) $ver
            }
         }
         catch {
            if { double($versions_http(BotikKey)) > double($botikkey(ver)) } {
               showHelp HelpDownload                              
               message info \
                            "Используется устаревшая версия программы: $botikkey(name) v.$botikkey(ver).\
                            \nНа сайте программы доступна новая версия:   $botikkey(name) v.$versions_http(BotikKey).\n\
                            \nСм. Справку"
            }
         }
      }
      default {
      }
   }
   return []
}

# Идея проверки единственности запущенного BKey-я состоит в том, что для
# каждого порта можно запустиь ровно один сервер. Работающий BKey держит
# сервер на порту $botikkey(port). Правда данный сервер весьма странный:
# он закрывает каждую попытку открыть с ним сессию (сокет).

proc checkSingleInvocationOfBotikKey { } {
   global botikkey
   set ok 0
   catch {
      socket -server newConnectionOnMyBKserver $botikkey(port)
      set ok 1
   }
   if { $ok } {
      return []   
   }
   message error "Один экземпляр Botik Key уже работает.\
                \nПовторный экземпляр сейчас будет завершен."
   exit
}

proc newConnectionOnMyBKserver {channel clientAddress clientPort} {
   catch {close $channel}
   return []   
}
