#!/bin/sh\
exec wish "$0" "$@"

# Copyright (c) 2002-2008, Botik-Technologies, Ltd.
# See license.txt for more information.

# ================= Log =========================================================
#
# Предоставляет
#
#     proc forceLogWindow   { }
#     proc showLogWindow   { }
#     proc writeLog        { level tag text },
#
#                     level = 0, 1, 2, ... -- подробность
#                     tag   = GRAY GREEN YELLOW RED BLUE
#
# Использует
#
#     botikkey(version)
#     botikkey(verbose)
#
# Параметры и переменные
#
global bkLog
set    bkLog(k0)     0 ;        # нижнее значение индекса с существующими записями в Log-е
set    bkLog(kN)    -1 ;        # верхнее значение индекса с существующими записями в Log-е,
                                # т.е. Log в bkLog(lines,$k), bkLog(tag,$k), bkLog(level,$k)
                                # $k = $bkLog(k0)...$bkLog(kN)
set    bkLog(MAX)   750 ;       # максимальное число записей, сохраняемых в Log-е  (750)
set    bkLog(MMAX) 1500 ;       # максимальное число записей, сохраняемых в Log-е (1500)
set    bkLog(minV)   0 ;        # нижний и 
set    bkLog(maxV)   3 ;        # верхний предел botikkey(verbose)
set    bkLog(width)  740 ;      # ширина окна
set    bkLog(chars)  80 ;       # ширина окна в колонках
set    bkLog(height) 400 ;      # высота окна
set    bkLog(bg)    snow2 ;     # background
set    bkLog(bgbtns) gray84 ;  # background кнопок
                                # тэги в окне
global iface
set    bkLog(tags) {
   BLUE    "-font {$iface(font3)} -background LightSkyBlue1"
   RED     "-font {$iface(font3)} -background RosyBrown1"
   YELLOW  "-font {$iface(font3)} -background LemonChiffon"
   GREEN   "-font {$iface(font3)} -background DarkSeaGreen2"
   GRAY    "-font {$iface(font4)}      -background azure1"
   SKIP    "-font {$iface(font6)}       -background gray86"
}
# ================= Log =========================================================

proc writeLog { level tag text } {
   global botikkey
   global bkLog

   set  kN [set bkLog(kN) [expr $bkLog(kN) + 1] ]

   set  bkLog(lines,$kN)  [set lines [splitWrapLines $bkLog(chars) $text] ]
   set  bkLog(tag,$kN)    $tag
   set  bkLog(level,$kN)  $level

   if { ([validateVerbose] >= $level) && ([winfo exists .wBkLog]) } {
      insertTextToWindow $level $lines $tag
   }

   # проверка переполнения и очистка:

   if { ($bkLog(kN) - $bkLog(k0)) < $bkLog(MMAX) } {
      return []
   }

   set k0_new [set bkLog(k0) [expr [set k0_old $bkLog(k0)] + $bkLog(MAX) ] ]
   refreshLog
   for {set k $k0_old } { $k <= $k0_new } { incr k } { catch {
       unset bkLog(lines,$kN)
       unset bkLog(tag,$kN)
       unset bkLog(level,$kN)
   }  }
   return []
}

proc insertTextToWindow { level lines tag } {
   global bkLog

   if { ([validateVerbose] < $level) || (! [winfo exists .wBkLog]) } {
      return []
   }
   catch {
      .wBkLog.cntnt.text configure -state normal
      foreach line $lines {
         .wBkLog.cntnt.text insert end "$line\n" $tag
      }
      .wBkLog.cntnt.text insert end " \n" SKIP
      .wBkLog.cntnt.text see end
      .wBkLog.cntnt.text configure -state disabled -cursor arrow
   }
   return []
}

proc insertLogToWindow { } {
   global bkLog

   set k0 $bkLog(k0)
   set kN $bkLog(kN)
   for {set k $k0 } { $k <= $kN } { incr k } {
      catch { 
         insertTextToWindow $bkLog(level,$k) $bkLog(lines,$k) $bkLog(tag,$k) 
      }
   }

   return []
}

proc refreshLog { } {
   catch {
      .wBkLog.cntnt.text configure -state normal
      .wBkLog.cntnt.text delete 1.0 end 
      .wBkLog.cntnt.text configure -state disabled -cursor arrow
      insertLogToWindow
      popUpLogWindow
   }
   return []
}

proc forceLogWindow {{msg {0}}} {
   global botikkey
   if {$msg || $botikkey(showLog) > 0} {
      showLogWindow
   }
   if {$msg} {bell}
   return []
}

proc showLogWindow { } {
   global bkLog

   if { [winfo exists .wBkLog] } {
      popUpLogWindow
      return []
   }

   createLogWindow
   insertLogToWindow
   popUpLogWindow

   return []
}

proc saveLog { } {
   global bkLog
   global botikkey
   global bkClientTimeOut

   set types {
       {{Text Files}       {.txt}        }
       {{All Files}        *             }
   }

   set tt [clock format [clock seconds] -format "%Y-%m-%d_%H_%M_%S"]
   set writeFileName "botik-key-log-$tt.txt"

   set filename [tk_getSaveFile -filetypes $types -defaultextension .txt \
                                -title "Сохранить журнал в файле?" \
                                -initialfile $writeFileName ]
   if {[string length $filename] == 0} {
      return []
   }

   set ok 0
   catch {
      set out [open $filename w]
      puts $out $botikkey(version)
      puts $out [getWhoAmIInfo]
      puts $out ":Server:$botikkey(addr)"
      puts $out ":Timeout:$bkClientTimeOut\n"

      set k0 $bkLog(k0)
      set kN $bkLog(kN)
      for {set k $k0 } { $k <= $kN } { incr k } { 
         catch { 
            foreach line $bkLog(lines,$k) {
               puts $out $line
      }  }  }
      close $out
      set ok 1
   } rc
   if { $ok } {
      message info "Журнал записан в файл\n\"$filename\""
   } else {
      catch { close $out }
      set msg "Не удалось записать журнал в файл\n\"$filename\"\nError: [mkErrorCode $rc]!"
      message error $msg
      asksendmail $msg
   }
   return []
}

proc popUpLogWindow { } {
   catch {wm deiconify .wBkLog; raise .wBkLog; focus -force .wBkLog}
   update
   return []
}

proc createLogWindow { } {
   global botikkey
   global bkLog
   global myImages
   global iface
   
   if { [winfo exists .wBkLog] } {
      return []
   }

   toplevel    .wBkLog -background $bkLog(bgbtns) -relief sunken -bd 4 -width $bkLog(width) -height $bkLog(height)
   #catch "wm iconbitmap .wBkLog $botikkey(currentIcon)"
   wm title    .wBkLog "Журнал программы $botikkey(version)"
   wm iconname .wBkLog "Log::$botikkey(version)"
   setWindowIcon .wBkLog

   labelframe .wBkLog.top -text "Журнал программы $botikkey(version)" -font $iface(font2) \
                          -background $bkLog(bgbtns) -width $bkLog(width)
   pack  .wBkLog.top -side top -fill x
   frame .wBkLog.top.bs    -width $bkLog(width) -background $bkLog(bgbtns) 
   frame .wBkLog.top.delim -width $bkLog(width) -height 0 -borderwidth 0 -background $bkLog(bgbtns)
   pack  .wBkLog.top.delim -side top -pady 0 -padx 0
   pack  .wBkLog.top.bs -side left -pady 0 -padx 0

   setImgButtons .wBkLog.top.bs 2 2 left w [subst { 
         help     $myImages(helpNormal)    {showHelp HelpForLog}
         save     $myImages(saveNormal)    {saveLog; popUpLogWindow}
         close    $myImages(noNormal)      {destroy .wBkLog}
         refresh  $myImages(refreshNormal) {refreshLog}
   } ]
   validateVerbose
   frame   .wBkLog.top.bs.fill
   pack    .wBkLog.top.bs.fill -side left -pady 2 -padx 0 -fill x
   label   .wBkLog.top.bs.vlabl -text "Уровень подробности показа журнала" -font $iface(font4)
   spinbox .wBkLog.top.bs.verbs -from $bkLog(minV) -to $bkLog(maxV) -increment 1 -width 2 -textvariable botikkey(verbose); # -validate all -vcmd {vCMD validateVerbose}

   pack .wBkLog.top.bs.vlabl .wBkLog.top.bs.verbs -side left -pady 2 -padx 6

   frame .wBkLog.cntnt -background $bkLog(bg) 
   pack  .wBkLog.cntnt -side top -fill both -expand yes
   text .wBkLog.cntnt.text -relief sunken -bd 2  -background $bkLog(bg) -font $iface(font4) \
        -yscrollcommand ".wBkLog.cntnt.scroll set" -setgrid 1 -height 30 -width 20 -wrap none
   scrollbar .wBkLog.cntnt.scroll -command ".wBkLog.cntnt.text yview"
   pack .wBkLog.cntnt.scroll -side right -fill y
   pack .wBkLog.cntnt.text -expand yes -fill both
   .wBkLog.cntnt.text configure -state disabled -cursor arrow

   set bkLog(M) 0
   foreach {name parms} $bkLog(tags) {
      eval [subst ".wBkLog.cntnt.text tag configure $name $parms -lmargin1  5 -lmargin2  5 -rmargin 5"]
   }
   changesInHelpWindow HelpForLog
   return []
}
