#!/bin/sh\
exec wish "$0" "$@"

# Copyright (c) 2002-2008, Botik-Technologies, Ltd.
# See license.txt for more information.

# ================= Main ========================================================
#
# Предоставляет
#
#     proc startMainWindow { }
#     proc changeStatus { uColorName text } -- uColorName: одно из {RED YELLOW GREEN}
#     proc resetIcons { }
#
# Использует
#
#     botikkey
#     myImages
#
# Параметры и переменные
#

proc startMainWindow { } {
   global botikkey
   global myImages
   global iface

   set icofile [file join . img botik_key.ico]
   if {[windowsPlatform]} {
      wm iconbitmap . -default $icofile
   }

   wm resizable . 0 0
   wm protocol . WM_DELETE_WINDOW {doExit}
   createIcon
   setWindowIcon .
   catch {destroy .mw}

   labelframe .mw -text $botikkey(version) -font $iface(font2)
   pack  .mw
   focus -force .mw

   frame .mw.top
   frame .mw.top.left
   frame .mw.top.right
   frame .mw.bottom
   pack  .mw.top        -side top    -padx 0 -pady 0 -anchor s -fill y
   pack  .mw.bottom     -side bottom -padx 3 -pady 0 -anchor s -fill y
   pack  .mw.top.left   -side left   -padx 3 -pady 0 -anchor w -fill y
   pack  .mw.top.right  -side right  -padx 3 -pady 3 -anchor e -fill both

   label .mw.top.left.lbl -image $myImages(YELLOW) -bd 1 -relief ridge
   pack .mw.top.left.lbl -side left -padx 3

   setImgButtons .mw.top.right 2 2 top n [subst { 
         help  $myImages(helpNormal)  {showHelp HelpForMainWindow}
         setup $myImages(setupNormal) runOptions
         log   $myImages(logNormal)   showLogWindow
   } ]
   tk_optionMenu .mw.bottom.region botikkey(region_by_user) WORLD LOCAL NONE
   pack .mw.bottom.region -side bottom -anchor s

   trace add variable botikkey(region_by_user) { write } newAccessRegion

   # Если окно сворачивается, то оно исчезает из поля зрения:
   if {[windowsPlatform] && !$::notray} {
      bind . <Unmap> "wm state . withdrawn"
      # При нажатии на кнопку Esc, главное окно BotikKey скрывается:
      bind . <Escape> [bind . <Unmap>]
   }

   setTitlesAndLights YELLOW "Init :: Ключ к СТ \"$::net(name)\" -- [strTime]"  "Init::Botik Key"

   changesInHelpWindow HelpForMainWindow
   
   return []
}

proc newAccessRegion { n1 n2 op } {
   global botikkey
   set region $botikkey(region_by_user)
   if { $botikkey(region_request) == $region } { return [] }
   set botikkey(region_request) $region
   writeLog 0 GRAY "[strTime] Изменен запрашиваемый доступ, теперь просим $region"
   if { $botikkey(client_started) } {
      set botikkey(client_started) 0   
      restartClient
   }
   return []
}

proc resetIcons { } {
   global botikkey
   setIconsAndLights $botikkey(currentUColorName)
   return []
}

# без побочного эффекта, вернет одно из {RED YELLOW GREEN GREEN_LOCAL}
proc checkColorName { colorName } {
   global botikkey
   if { [set i [lsearch -exact { RED YELLOW GREEN GREEN_LOCAL } $colorName]] < 0 } {
      return YELLOW
   }
   if {($i == 2) && ($botikkey(region_by_server) eq "LOCAL")} {
      return GREEN_LOCAL
   }
   return $colorName
}

proc setIconsAndLights {uColorName} {
   global botikkey
   #dolog "Иконки и цвета устанавливаем в $uColorName,"
   set colorName [checkColorName $uColorName] ; # RED | YELLOW | GREEN | GREEN_LOCAL 
   set newIcon   [lindex $botikkey($colorName) [validateIconType]]
   configImage $colorName
   #dolog "сконфигурировали картинку $colorName,"
   if {$botikkey(currentIcon) ne $newIcon} {
      set botikkey(currentUColorName) $colorName
      set botikkey(currentIcon)       $newIcon
      #dolog "заменяем иконку в трее на $newIcon..."
      trayIcon modify
      #dolog "иконка заменена,"
   }
   #dolog "текущая иконка в трее - $botikkey(currentIcon),"
   # Проблема BotikKey была в том, что велась работа с несуществующими окнами:
   # изменение иконок, заглавий. В этой версии проблема решается.
   foreach {w} {. .wBkLog .wHelp .wBkOptions} {
       if {[winfo exists $w] && $w ne "."} {
           if {![winfo ismapped $w]} {
            wm deiconify $w
            raise $w
        }
           #dolog "меняем иконку окна $w..."
           setWindowIcon $w
           #dolog "иконка в окне $w заменена,"
       } elseif {$w eq "." && [winfo ismapped $w]} {
           #dolog "меняем иконку окна $w..."
           setWindowIcon $w
           #dolog "иконка в окне $w заменена,"
       }
   }
   #dolog "заменили все иконки в окнах."
   return []
}

proc configImage {colorName} {
   global myImages
   .mw.top.left.lbl configure -image $myImages($colorName)
}

proc setTitlesAndLights {uColorName t ti} {
   setIconsAndLights $uColorName
   #dolog "Замена иконок и цветов завершена."
   set ::roottitles [list $t $ti]
   if {[winfo ismapped .]} {
       #dolog "Начинаем замену заголовков,"
       wm title . $t
       #dolog "заголовок корневого окна заменен на \"$t\","
       wm iconname . $ti
       #dolog "имя иконки корневого окна заменено на \"$ti\","
       update
       #dolog "выход."
   }
   update
   return []
}
proc initMenu {} {
      global botikkey
      catch {destroy .m}
      update
      if {[winfo exists .m]} {return}
      menu .m -tearoff 0
      .m add command -label "Справка" -command "showHelp HelpForMainWindow"
      .m add command -label "Настройки" -command "runOptions"
      .m add command -label "Журнал" -command "showLogWindow"
      .m add separator
      foreach {region name} $botikkey(modenames) {
          if {$region ne $botikkey(region_by_user)} {
          .m add command -label $name -command "set botikkey(region_by_user) $region"
        }
      }
      .m add separator
      if {![windowsPlatform]} {
	      .m add command -label "Закрыть меню" -command "catch {destroy .m}"
	      .m add separator
      }
      .m add command -label "Выход" -command "doExit"
}
proc tray_cmd {winmsg x y} {
    if {$winmsg eq "WM_LBUTTONDBLCLK"} {
        #dolog "Левый двойной клик по иконке в трее,"
        set ::botikkey(region_by_user) [expr {($::botikkey(region_by_user) eq "WORLD") ? "LOCAL" : "WORLD"}]
        #dolog "режим доступа изменён на $::botikkey(region_by_user)."
    } elseif {$winmsg eq "WM_LBUTTONUP"} {
        #dolog "Левый клик по иконке в трее,"
        # Левый клик по иконке в трее: если окно свёрнуто, то оно разворачивается;
        # если развёрнуто, то сворачивается.
        if {![winfo ismapped .]} {
            wm deiconify .
            raise .
            focus -force .
            #dolog "корневое окно восстановлено,"
            # меняем иконку корневого окна
               #dolog "меняем иконку корневого окна..."
               setWindowIcon .
               #dolog "иконка корневого окна заменена."
            if {[info exists ::roottitles]} {
                set t  [lindex $::roottitles 0]
                set ti [lindex $::roottitles 1]
                #dolog "Начинаем замену заголовков,"
                wm title . $t
                #dolog "заголовок корневого окна заменен на \"$t\","
                wm iconname . $ti
                #dolog "имя иконки корневого окна заменено на \"$ti\","
                update
                #dolog "выход."
            }
        } else {
            wm state . withdrawn
            #dolog "корневое окно скрыто."
        }
    } elseif {$winmsg eq "WM_RBUTTONUP"} {
        # Правый клик по иконке в трее: появляется меню.
        #dolog "Правый клик по иконке в трее,"
        # Создание меню:
        initMenu
        .m post $x $y
        #.m activate 0
        #dolog "появилось меню."
    }
}
proc warning {} {
   set ans [tk_messageBox  -type okcancel -icon warning -title "Предупреждение" -parent .mw \
      -message "Вы завершаете работу BotikKey. Ваш компьютер потеряет\nдоступ к Сети (вплоть до следующего запуска BotikKey)."]
   return $ans
}
proc delete_icons {} {
	if {$::notray} {return}
    if {![windowsPlatform]} {return}
    catch {winico delete [winico info]}
    update
    #dolog "Иконка в трее удалена."
}
proc doExit {} {
    if {[warning] eq "ok"} {delete_icons; exit}
}
proc changeStatus { uColorName text } {
    #dolog "Изменение статуса на $uColorName - $text,"
    global botikkey
    set cName [checkColorName $uColorName]
    switch $cName {
        RED    {
            setTitlesAndLights RED "NONE! Ключ к СТ \"$::net(name)\" -- [strTime]"  "NONE::Botik Key"
        }
        YELLOW {
            setTitlesAndLights YELLOW "Ждем. Ключ к СТ \"$::net(name)\" -- [strTime]" "Waiting::Botik Key"
        }
        GREEN -
        GREEN_LOCAL {
            set region $botikkey(region_by_server)
            setTitlesAndLights GREEN  "$region! Ключ к СТ \"$::net(name)\" -- [strTime]" "${region}::Botik Key"
            if {$botikkey(chk_version)} {
                #dolog "проверяется версия..."
                catch {readURL $botikkey(url_chk_version) 10000}
                #dolog "версия проверена,"
                set botikkey(chk_version) 0
            }
        }
    }
    #dolog "изменение статуса завершено."
    return []
}
proc findTraybarIconPos {} {
    global botikkey positions
    if {$::notray} {return}
    set ico         [winico info]
    set info        {}
    catch {set info [winico info $ico]}
    if {$info eq {}} {return -1}
    set screendepth [winfo screendepth .]
    set size        "16x16"
    set pos         -1
    set bigdepth    0
    for {set j [expr 9*$positions($botikkey(currentIcon))]} {$j < [expr 9*(1+$positions($botikkey(currentIcon)))]} {incr j} {
        # для каждой иконки есть 9 разновидностей, среди них ищем подходящую по размеру и по глубине цвета
        set i [lindex $info $j]
        array set opts $i
        if {![info exists opts(-bpp)] || ![info exists opts(-pos)] || ![info exists opts(-geometry)]} {return -1}
        set depth    $opts(-bpp)
        set pos      $opts(-pos)
        set geometry $opts(-geometry)
        if {$geometry eq "$size" && $depth <= $screendepth} {
            if {$depth > $bigdepth} {
                set pos      $pos
                set bigdepth $depth
            }
        }
    }
    return $pos
}
proc findTaskbarIconsPos {} {
    global botikkey positions
    if {$::notray} {return {0 0}}
    if {[windowsPlatform]} {
    	set ico [winico info]
    } else {
    	set ico ""
    }
    set info        {}
    catch {set info [winico info $ico]}
    if {$info eq {}} {return [list -1 -1]}
    set screendepth [winfo screendepth .]
    set smallsize   "16x16"
    set bigsize     "32x32"
    set smallpos    -1
    set bigpos      -1
    set smalldepth  0
    set bigdepth    0
    for {set j [expr 9*$positions($botikkey(currentIcon))]} {$j < [expr 9*(1+$positions($botikkey(currentIcon)))]} {incr j} {
        # для каждой иконки есть 9 разновидностей, среди них ищем подходящие по размерам и по глубине цвета
        set i [lindex $info $j]
        array set opts $i
        if {![info exists opts(-bpp)] || ![info exists opts(-pos)] || ![info exists opts(-geometry)]} {return [list -1 -1]}
        set depth    $opts(-bpp)
        set pos      $opts(-pos)
        set geometry $opts(-geometry)
        if {$geometry eq "$bigsize" && $depth <= $screendepth} {
            if {$depth > $bigdepth} {
                set bigpos   $pos
                set bigdepth $depth
            }
        } elseif {$geometry eq "$smallsize" && $depth <= $screendepth} {
            if {$depth > $smalldepth} {
                set smallpos   $pos
                set smalldepth $depth
            }
        }
    }
    return [list $smallpos $bigpos]
}
proc setWindowIcon {w} {
    # только если окно развёрнуто:
    if {[winfo exists $w] && [winfo ismapped $w]} {
        update
        #dolog "Обновленo (1),"
    }
    #dolog "поиск нужных иконок,"
    foreach {smallpos bigpos} [findTaskbarIconsPos] {}
    if {!$::notray && [winfo exists $w] && [winfo ismapped $w]} {
        if {$smallpos >= 0} {
            catch {winico setwindow $w [winico info] "small" $smallpos}
        }
        #dolog "малая иконка заменена,"
    }
    if {!$::notray && [winfo exists $w] && [winfo ismapped $w]} {
        if {$bigpos >= 0} {
            catch {winico setwindow $w [winico info] "big" $bigpos}
        }
        #dolog "большая иконка заменена,"
    }
    if {[winfo exists $w] && [winfo ismapped $w]} {
        update
        #dolog "обновленo (2)."
    }
}
proc createIcon {} {
    #dolog "Создаётся иконка в трее,"
    if {$::notray} {return}
    catch {winico createfrom "img/botik.ico"}
    trayIcon add
    #dolog "иконка в трее создана."
}
proc trayIcon {cmd} {
    global botikkey
    if {$::notray} {return}
    array set names $botikkey(modenames)
    if {[windowsPlatform]} {
	    set pos 0
	    catch {set pos [lindex [findTraybarIconPos] 0]}
	    if {$cmd eq "add"} {
	        if {$pos >= 0} {
	            catch {winico taskbar add [winico info] -callback "tray_cmd %m %x %y" -text $names($botikkey(region_by_user)) -pos $pos}
	        }
	    } else {
	        if {$pos >= 0} {
	            catch {winico taskbar modify [winico info] -text $names($botikkey(region_by_user)) -pos $pos}
	        }
    	}
    } else {
        if {$cmd eq "add"} {
            image create photo ico -file $botikkey(currentIcon)
            tktray::icon $botikkey(iconwindow) -image ico -visible true
            bind $botikkey(iconwindow) <ButtonRelease-1> "tray_cmd WM_LBUTTONUP %X %Y"
            bind $botikkey(iconwindow) <Double-ButtonPress-1> "tray_cmd WM_LBUTTONDBLCLK %X %Y"
            bind $botikkey(iconwindow) <ButtonRelease-3> "tray_cmd WM_RBUTTONUP %X %Y"
        } else {
            image create photo ico -file $botikkey(currentIcon)
            $botikkey(iconwindow) configure -image ico
         }
    }
}
