#!/bin/sh\
exec wish "$0" "$@"

# Copyright (c) 2002-2008, Botik-Technologies, Ltd.
# See license.txt for more information.

package require Tcl  8.2
package require md5  2.0.0

# ===========================================================================================
proc mkErrorCode { rc {delim "\n"}} {
   global errorCode
   set x $errorCode
   set t [lindex $x 0]
   switch $t {
      ARITH       -
      CHILDKILLED -
      CHILDSUSP   - 
      POSIX       { return "[lindex $x end]$delim$rc" }
      default     { return $rc }
   }
}

proc loadModule { fname } {
   global myModules
   if [info exists myModules($fname)] {
      tk_messageBox -icon warning -type ok -message "Модуль \"$fname\" уже загружен"
      return []
   }
   set ok 0
   catch {
      source $fname
      set ok 1
      set myModules($fname) 1
   } rc
   if { ! $ok } {
      tk_messageBox -icon error -type ok -message \
            "Не удалось загрузить модуль программы \"$fname\"\nError: [mkErrorCode $rc]\n\
            \nВыполнение программы сейчас будет завершено.\n\
            \nСоветуем попробовать переустановить программу."
      exit
   }
   return []
}

proc loadImage { name fname } {
   global myImages
   if [info exists myImages($name)] {
      return []
   }
   set ok 0
   catch {
      set myImages($name) [image create photo -file $fname]
      set ok 1
   } rc
   if { ! $ok } {
      tk_messageBox -icon error -type ok -message \
            "Не удалось загрузить графический файл для программы: \"$fname\"\nError: [mkErrorCode $rc]\n\
            \nВыполнение программы сейчас будет завершено.\n\
            \nСоветуем попробовать переустановить программу."
      exit
   }
   return []
}

proc debug { str } {
   message info "DEBUG $str"
}

# ===========================================================================================
loadModule bkInit.tcl
loadModule botikUts.tcl
loadModule net.tcl
loadModule report.tcl

loadImage  fnameNormal       img/pswd_write.gif
loadImage  noNormal          img/no.gif

# ================= Собственно скрипт =============================================

proc mkKey { } {
   after 10 mkKey1
   return 1
}

proc mkKey1 { } {
   global pswd
   global key
   set key [pswdToKey  $pswd]
   return []
}

proc writeKey { } {
   global pswd
   global key

   set key  [pswdToKey  $pswd]

   set types {
       {{Text Files}       {.txt}        }
       {{All Files}        *             }
   }

   set writeFileName "*.txt"

   set filename [tk_getSaveFile -filetypes $types -defaultextension .txt -parent .mw \
                                -title "Сохранить ключ доступа в файле?" \
                                -initialfile $writeFileName ]

   if { ([string length $filename] == 0) || ([string first "*" $filename] >=0)} {
      message info "Ключ доступа не был записан в файл"
      return []
   }

   # $filename -- имя файла
   set goon "retry"
   set ok 0
   while {$goon eq "retry"} {
      set ok 0
      catch {
         # запись пароля в файл $filename
           set out [open $filename w]
         puts $out $key
         close $out
         set ok 1
      } rc
      if { !$ok } {
         catch { close $out }
         set msg "Не удалось записать ключ доступа в файл\n\"$filename\"\n\n[mkErrorCode $rc]"
         set goon [ tk_messageBox -icon warning -type retrycancel -default retry \
                                  -title "Ошибка записи ключа доступа в файл" -parent .mw \
                                  -message $msg ]
         asksendmail $msg
      } else {
         set goon cancel
      }
   }
   if { !$ok } {
      message info "Ключ доступа не был записан в файл"
      return []
   }
   message info "Ключ доступа был записан в файл:\n$filename"
   return []
}
proc genPw {{pwlen 8}} {
# Генератор криптографически-надежных паролей.
# Базируется на генераторе случайных чисел ISAAC.
# В качестве зерна используется путь движения курсора мыши по области окна.
	package require Random 1.2.0
    if {![info exists ::track] || $::track eq {}} {return}
    if {![info exists ::seed]} {
        append ::seed [clock clicks] [winfo id .] $::track
        random::isaac seed $::seed
        bind . <Motion> ""
        set ::seed {}
    }

    set chars [split {aAbBcCdDeEfFgGhHiIjJkKlLmMnNpPqQrRsStTuUvVwWxXyYzZ123456789} ""]; # Список возможных символов пароля; "O" и "0" не включены.
    set len   [llength $chars]
    set pw    {}

    while {[string length $pw] < $pwlen} {
        set rand [expr abs([random::isaac integer])]
        while {$rand > 0 && $pwlen && [string length $pw] < $pwlen} {
            set rest [expr {$rand % $len}]
            set rand [expr {$rand / $len}]
            append pw [lindex $chars $rest]
        }
    }
    set re1 {[a-z]+}
    set re2 {[A-Z]+}
    set re3 {[1-9]+}
    if {[regexp $re1 $pw]&&[regexp $re2 $pw]&[regexp $re3 $pw]} {
    	return $pw
    } else {
    	return [genPw]
	}
}
# mouse tracker
proc addCoords {x y} {
    append ::track $x $y
}
proc newPw {} {
    set ::pswd [genPw]
    mkKey
    .mw.e1 icursor end
}

global pswd key botikkey tcl_platform track
set track {}; # mouse track

initValues
catch {destroy .mw}

catch {
   if { $tcl_platform(platform) == "windows" } {
      set icon img/botik_kg.ico
   } else {
      set icon [lindex $botikkey(YELLOW) 0 ]
   }
   wm  iconbitmap . $icon
   wm  title      . "Построение ключа доступа по заданному паролю"
}

labelframe .mw -text $botikkey(version) -font {Helvetica 10 bold}
pack  .mw
focus -force .mw
label .mw.t1 -font {Helvetica 10} -justify left -text \
                   "Это вспомогательная программа для $botikkey(version).\
                   \n\
                   \nДанная программа позволяет перевести пароль доступа в ключ доступа\
                   \n(читайте на сайте документацию об этих двух понятиях).\
                   \n\
                   \nПеревод пароля доступа в ключ доступа:\
                   \n\
                   \n1. Убедитесь, что потенциальный злоумышленник не подсматривает за Вами,\
                   \nчто нет угрозы удаленного перехвата ввода с клавиатуры или изображения\
                   \nс экрана."

frame  .mw.f
label  .mw.f.l -font {Helvetica 10} -justify left -text "\n2. Введите пароль в поле ввода здесь:"
button .mw.f.rpw -text "Случайный пароль" -command {newPw} -takefocus 0

set pswd [genPw]
mkKey1

entry .mw.e1 -validate all -vcmd mkKey -textvariable pswd -state normal -width 75

label .mw.t2 -font {Helvetica 10} -justify left -text \
                  "\n3. Здесь ключ доступа, построенный по введенному паролю доступа\
                  \n(этот ключ можно копировать при помощи мыши):"

entry .mw.e2 -textvariable key -state readonly -width 75

label .mw.t3 -font {Helvetica 10} -justify left -text \
                  "\n4. При помощи этих двух кнопок можно закрыть программу (левая кнопка)\
                  \nили записать построенный ключ в файл. Мы рекомендуем использовать только\
                  \nсъемные носители (дискеты, CD-диски, FLASH/USB-диски и т.п.), хранить их\
                  \nв надежном месте и отключать носитель от компьютера сразу, как только\
                  \nзавершена работа с ним (то есть, завершена запись или чтение ключа)."

frame .mw.bs

pack .mw.t1 -side top -anchor w -padx 5
pack .mw.f -side top -anchor w
pack .mw.f.l -anchor w -side left -padx 5
pack .mw.f.rpw -side left -padx 20 -anchor s -pady 3
pack .mw.e1 .mw.t2 .mw.e2 .mw.t3 .mw.bs -side top -anchor w -padx 5

setImgButtons .mw.bs 2 2 left w [subst { 
      close $myImages(noNormal)    {exit}
      save  $myImages(fnameNormal) writeKey
} ]

bind . <Motion> "addCoords %x %y"
focus -force .mw.e1
update
